from collections import OrderedDict
import numpy as np
import pandas as pd
import talib as ta
from loguru import logger
from czsc.connectors import research
from czsc import CZSC, check_signals_acc, get_sub_elements
from czsc.utils import create_single_signal


def obv_up_dw_line_V230719(c: CZSC, **kwargs) -> OrderedDict:
    """OBV能量指标，贡献者：琅盎

    参数模板："{freq}_D{di}N{n}M{m}MO{max_overlap}_OBV能量V230719"

    **信号逻辑：**

    OBV 指标把成交量分为正的成交量（价格上升时的成交量）和负的成交量（价格下降时）的成交量。
    OBV 就是分了正负之后的成交量的累计和。

    1. 先定义OBVM，OBVM是 OBV 7天的指数平均。
    2. 再定义一条信号线Signal line，这条线是OBVM 10天的指数平均。

    其中的「7天」和「10天」都是参数，根据你的交易时间级别设置，设置的越小，OBVM对成交量的变化越敏感，
    产生的交易信号也就越多。

    开多仓的规则：当OBVM上穿Signal line，开多仓；当OBVM下穿Signal line，平仓。这个规则只捕捉上涨趋势。

    **信号列表：**

    - Signal('日线_D1N7M10MO3_OBV能量V230719_看多_任意_任意_0')
    - Signal('日线_D1N7M10MO3_OBV能量V230719_看空_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 参数字典
    
        - :param di: 信号计算截止倒数第i根K线
        - :param n: short窗口大小。
        - :param m: long窗口大小。
        - :param max_overlap: 信号计算时，最大重叠K线数量。
        
    :return: 信号识别结果
    """
    di = int(kwargs.get("di", 1))
    n = int(kwargs.get("n", 7))
    m = int(kwargs.get("m", 10))
    max_overlap = int(kwargs.get("max_overlap", 3))
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_D{di}N{n}M{m}MO{max_overlap}_OBV能量V230719".split('_')
    v1 = "其他"

    # 计算OBV，缓存到bar.cache中
    cache_key = "OBV"
    for i in range(1, len(c.bars_raw)):
        bar1, bar2 = c.bars_raw[i - 1], c.bars_raw[i]
        if cache_key not in bar1.cache:
            last_obv = bar1.vol if bar1.close > bar1.open else -bar1.vol
            bar1.cache[cache_key] = last_obv
        else:
            last_obv = bar1.cache[cache_key]

        if cache_key not in bar2.cache:
            cur_obv = bar2.vol if bar2.close > bar2.open else -bar2.vol
            bar2.cache[cache_key] = last_obv + cur_obv

    min_k_num = di + max(n, m) + max_overlap + 10
    if len(c.bars_raw) < min_k_num:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = get_sub_elements(c.bars_raw, di=di, n=min_k_num)
    obv_seq = np.array([x.cache[cache_key] for x in bars], dtype=np.float64)
    obvm = ta.EMA(obv_seq, n)
    sig_ = ta.EMA(obvm, m)

    if obvm[-1] > sig_[-1] and obvm[-max_overlap] < sig_[-max_overlap]:
        v1 = "看多"
    elif obvm[-1] < sig_[-1] and obvm[-max_overlap] > sig_[-max_overlap]:
        v1 = "看空"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def main():
    symbols = research.get_symbols('A股主要指数')
    bars = research.get_raw_bars(symbols[0], '15分钟', '20191101', '20210101', fq='前复权')

    signals_config = [
        {'name': obv_up_dw_line_V230719, 'freq': '日线', 'di': 1},
        # {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 1},
    ]
    check_signals_acc(bars, signals_config=signals_config) # type: ignore


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2022/2/16 20:31
describe: czsc.utils 单元测试
"""
import pytest
import pandas as pd
from czsc import utils


def test_x_round():
    assert utils.x_round(100, 3) == 100
    assert utils.x_round(1.000342, 3) == 1.0
    assert utils.x_round(1.000342, 4) == 1.0003
    assert utils.x_round(1.000342, 5) == 1.00034


def test_subtract_fee():
    from czsc.utils.stats import subtract_fee

    # 构造测试数据
    data = {
        'dt': pd.date_range('2022-01-01', periods=20, freq='D'),
        'pos': [0, 1, 1, -1, -1, -1, 0, 0, 1, 1, 1, 1, 1, 0, 0, -1, -1, 0, 1, 1],
        'price': [10, 11, 12, 13, 14, 10, 11, 12, 13, 14, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
    }
    df = pd.DataFrame(data)

    # 执行函数
    df = subtract_fee(df, fee=100)

    # 验证结果
    assert int(df['edge_pre_fee'].sum()) == 2748
    assert int(df['edge_post_fee'].sum()) == 1848

    # 构造测试数据
    data = {
        'dt': pd.date_range('2022-01-01', periods=5, freq='D'),
        'pos': [0, 1, 1, -1, 0],
    }
    df = pd.DataFrame(data)

    # 执行函数并捕获异常
    with pytest.raises(AssertionError):
        subtract_fee(df, fee=1)

    # 构造测试数据
    data = {
        'dt': pd.date_range('2022-01-01', periods=5, freq='D'),
        'pos': [0, 1, 2, -1, 0],
        'price': [10, 11, 12, 13, 14]
    }
    df = pd.DataFrame(data)

    # 执行函数并捕获异常
    with pytest.raises(AssertionError):
        subtract_fee(df, fee=1)


from collections import OrderedDict
import numpy as np
import pandas as pd
import talib as ta
from loguru import logger
from czsc.connectors import research
from czsc import CZSC, check_signals_acc, get_sub_elements
from czsc.utils import create_single_signal


def obvm_line_V230610(c: CZSC, **kwargs) -> OrderedDict:
    """OBV能量指标，贡献者：琅盎

    参数模板："{freq}_D{di}N{n}M{m}_OBV能量V230610"

    **信号逻辑：**

    OBV 指标把成交量分为正的成交量（价格上升时的成交量）和负的
    成交量（价格下降时）的成交量。OBV 就是分了正负之后的成交量
    的累计和。

    首先，根据传入的参数 di、n 和 m，从 CZSC 对象中获取对应的 K 线数据，然后计算 OBV 序列。
    接着，使用 talib 库中的 EMA 函数计算 OBV 序列的短期和长期指数移动平均线，
    最后根据两条移动平均线的大小关系判断看多或看空信号。
    
    **信号列表：**

    - Signal('日线_D1N10M30_OBV能量V230610_看空_任意_任意_0')
    - Signal('日线_D1N10M30_OBV能量V230610_看多_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 参数字典
    
        - :param di: 信号计算截止倒数第i根K线
        - :param n: short窗口大小。
        - :param m: long窗口大小。
        
    :return: 信号识别结果
    """
    di = int(kwargs.get("di", 1))
    n = int(kwargs.get("n", 10))
    m = int(kwargs.get("m", 30))
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_D{di}N{n}M{m}_OBV能量V230610".split('_')
    v1 = "其他"

    # 计算OBV，缓存到bar.cache中
    cache_key = "OBV"
    for i in range(1, len(c.bars_raw)):
        bar1, bar2 = c.bars_raw[i - 1], c.bars_raw[i]
        if cache_key not in bar1.cache:
            last_obv = bar1.vol if bar1.close > bar1.open else -bar1.vol
            bar1.cache[cache_key] = last_obv
        else:
            last_obv = bar1.cache[cache_key]

        if cache_key not in bar2.cache:
            cur_obv = bar2.vol if bar2.close > bar2.open else -bar2.vol
            bar2.cache[cache_key] = last_obv + cur_obv

    if len(c.bars_raw) < di + max(n, m) + 10:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = get_sub_elements(c.bars_raw, di=di, n=max(n, m) + 10)
    obv_seq = np.array([x.cache[cache_key] for x in bars], dtype=np.float64)

    ema_n1 = ta.EMA(obv_seq, n)[-1]
    ema_n2 =ta.EMA(obv_seq, m)[-1]

    v1 =  "看多" if ema_n1 > ema_n2 else "看空"
    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def main():
    symbols = research.get_symbols('A股主要指数')
    bars = research.get_raw_bars(symbols[0], '15分钟', '20191101', '20210101', fq='前复权')

    signals_config = [
        {'name': obvm_line_V230610, 'freq': '日线', 'di': 1},
        # {'name': 'czsc.signals.tas_ma_base_V221101', 'freq': '日线', 'di': 1},
    ]
    check_signals_acc(bars, signals_config=signals_config) # type: ignore


if __name__ == '__main__':
    main()


from collections import OrderedDict
import numpy as np
import pandas as pd

from czsc.connectors import research
from czsc import CZSC, check_signals_acc, get_sub_elements
from czsc.utils import create_single_signal


def cvolp_up_dw_line_V230612(c: CZSC, **kwargs) -> OrderedDict:
    """CVOLP动量变化率指标，贡献者：琅盎

    参数模板："{freq}_D{di}N{n}M{m}UP{up}DW{dw}_CVOLP动量变化率V230612"

    **信号逻辑：**

    成交量移动平均平滑变化率。
    先计算了成交量的N周期指数移动平均线，然后计算了EMAP的M周期前的值，最后计算了CVOLP的值。
    CVOLP 上穿 up 买入，下穿 dw 卖出。

    **信号列表：**

    - Signal('日线_D1N13M21UP5DW5_CVOLP动量变化率V230612_看多_任意_任意_0')
    - Signal('日线_D1N13M21UP5DW5_CVOLP动量变化率V230612_看空_任意_任意_0')

    :param c: CZSC对象
    :param kwargs: 参数字典
        - :param di: 信号计算截止倒数第i根K线
        - :param n: 取K线数量
        - :param m: 信号预警值
        - :param up: 看多信号预警值
        - :param dw: 看空信号预警值
    :return: 信号识别结果
    """
    di = int(kwargs.get("di", 1))
    n = int(kwargs.get("n", 34))
    m = int(kwargs.get("m", 55))
    up = int(kwargs.get("up", 5))
    dw = int(kwargs.get("dw", 5))
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_D{di}N{n}M{m}UP{up}DW{dw}_CVOLP动量变化率V230612".split('_')

    # 增加一个约束，如果K线数量不足时直接返回
    v1 = "其他"
    if len(c.bars_raw) < di + n + 10:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)

    bars = get_sub_elements(c.bars_raw, di=di, n=n + m) 

    volume = np.array([bar.vol for bar in bars])
    n_weights = np.exp(np.linspace(-1., 0., n))
    n_weights /= n_weights.sum()
    emap = np.convolve(volume, n_weights, mode='full')[:len(volume)]
    emap[:n] = emap[n]
    sroc = (emap - np.roll(emap, m))[-1] / np.roll(emap, m)[-1] 

    if sroc > up / 100:
        v1 = "看多"
    if sroc < -dw / 100:
        v1 = "看空"

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def main():
    symbols = research.get_symbols('A股主要指数')
    bars = research.get_raw_bars(symbols[0], '15分钟', '20171101', '20210101', fq='前复权')

    signals_config = [
        {'name': cvolp_up_dw_line_V230612, 'freq': '日线', 'di': 1},
    ]
    check_signals_acc(bars, signals_config=signals_config)


if __name__ == '__main__':
    main()


from loguru import logger

try:
    import talib as ta
except:
    logger.warning(f"ta-lib 没有正确安装，相关信号函数无法正常执行。" f"请参考安装教程 https://blog.csdn.net/qaz2134560/article/details/98484091")
import math
import pandas as pd
import numpy as np
from collections import OrderedDict
from deprecated import deprecated
from czsc.analyze import CZSC
from czsc.objects import Signal, Direction, BI, RawBar, FX
from czsc.signals.tas import update_ma_cache
from czsc.utils import get_sub_elements, fast_slow_cross, count_last_same, create_single_signal
from czsc.utils.sig import cross_zero_axis, cal_cross_num, down_cross_count
from typing import Union,List



def tas_rumi_V230704(c: CZSC, **kwargs) -> OrderedDict:
    """对均线偏离度平滑处理,通过平滑处理的方式降低DIFF的敏感度来解决均线缠绕的问题 贡献者：谌意勇

    参数模板："{freq}_D{di}F{timeperiod1}S{timeperiod2}R{rumi_window}_BS辅助V230704"

    **信号逻辑：**

    RUMI 计算参考：
    1. https://zhuanlan.zhihu.com/p/610377004
    2. https://zhuanlan.zhihu.com/p/618394552

    多空规则：
    1. RUMI上穿0轴，买入做多
    2. RUMI下穿0轴，卖出做空。

    **信号列表：**

    - Signal('60分钟_D1F3S50R30_BS辅助V230704_空头_任意_任意_0')
    - Signal('60分钟_D1F3S50R30_BS辅助V230704_多头_任意_任意_0')

    :param c: CZSC对象
    :param kwargs:

        - di: 信号计算截止倒数第i根K线
        - timeperiod1: 均线1的周期
        - timeperiod2: 均线2的周期
        - rumi_window: rumi的周期

    :return: 信号识别结果
    """
    di = int(kwargs.get('di', 1))
    rumi_window = int(kwargs.get('rumi_window', 30))
    timeperiod1 = int(kwargs.get('timeperiod1', 3))
    timeperiod2 = int(kwargs.get('timeperiod2', 50))
    
    assert rumi_window < timeperiod2, "rumi_window 必须小于 timeperiod2"
    freq = c.freq.value
    k1, k2, k3 = f"{freq}_D{di}F{timeperiod1}S{timeperiod2}R{rumi_window}_BS辅助V230704".split('_')
    v1 = '其他'
    
    if len(c.bars_raw) < di + timeperiod2:
        return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)
     
    key1 = update_ma_cache(c, ma_type='SMA', timeperiod=timeperiod1)
    key2 = update_ma_cache(c, ma_type='WMA', timeperiod=timeperiod2)
    bars = get_sub_elements(c.bars_raw, di=di, n=timeperiod2)
    fast_array = np.array([x.cache[key1] for x in bars])
    slow_array = np.array([x.cache[key2] for x in bars])
    diff_array = fast_array - slow_array
    rumi_array = ta.MA(diff_array, timeperiod=rumi_window, matype=ta.MA_Type.SMA)

    if rumi_array[-1] > 0 and rumi_array[-2] < 0:
        v1='多头'
    elif rumi_array[-1] < 0 and rumi_array[-2] > 0:
        v1='空头'

    return create_single_signal(k1=k1, k2=k2, k3=k3, v1=v1)


def check():
    from czsc.connectors import research
    from czsc.traders.base import check_signals_acc

    symbols = research.get_symbols('A股主要指数')
    bars = research.get_raw_bars(symbols[0], '15分钟', '20181101', '20210101', fq='前复权')

    signals_config = [{'name': tas_rumi_V230704, 'freq': "60分钟", 'di': 1}]
    check_signals_acc(bars, signals_config=signals_config, height='780px', delta_days=0) # type: ignore


if __name__ == '__main__':
    check()
